#!/usr/bin/env python

import unittest

from plover_stroke import BaseStroke, stroke_integer_type


class StrokeSetupTestCase(unittest.TestCase):

    def test_setup_explicit(self):
        class Stroke(BaseStroke):
            pass
        keys = '''
            #
            S- T- K- P- W- H- R-
            A- O-
            *
            -E -U
            -F -R -P -B -L -G -T -S -D -Z
        '''.split()
        implicit_hyphen_keys = 'A- O- * -E -U'.split()
        Stroke.setup(keys, implicit_hyphen_keys)
        self.assertEqual(Stroke.KEYS, tuple(keys))
        self.assertEqual(Stroke.KEYS_IMPLICIT_HYPHEN, set(implicit_hyphen_keys))
        self.assertEqual(Stroke.KEYS_LETTERS, ''.join(keys).replace('-', ''))
        self.assertEqual(Stroke.KEY_FIRST_RIGHT_INDEX, keys.index('-E'))

    def test_setup_implicit_hyphens(self):
        class Stroke(BaseStroke):
            pass
        keys = '''
            #
            S- T- K- P- W- H- R-
            A- O-
            *
            -E -U
            -F -R -P -B -L -G -T -S -D -Z
        '''.split()
        Stroke.setup(keys)
        self.assertEqual(Stroke.KEYS, tuple(keys))
        self.assertEqual(Stroke.KEYS_IMPLICIT_HYPHEN, {'A-', 'O-', '*',
                                                       '-E', '-U', '-F'})
        self.assertEqual(Stroke.KEYS_LETTERS, ''.join(keys).replace('-', ''))
        self.assertEqual(Stroke.KEY_FIRST_RIGHT_INDEX, keys.index('-E'))

    def test_setup_numbers(self):
        class Stroke(BaseStroke):
            pass
        keys = '''
            #
            S- T- K- P- W- H- R-
            A- O-
            *
            -E -U
            -F -R -P -B -L -G -T -S -D -Z
        '''.split()
        implicit_hyphen_keys = 'A- O- * -E -U'.split()
        number_key = '#'
        numbers = {
            'S-': '1-',
            'T-': '2-',
            'P-': '3-',
            'H-': '4-',
            'A-': '5-',
            'O-': '0-',
            '-F': '-6',
            '-P': '-7',
            '-L': '-8',
            '-T': '-9',
        }
        Stroke.setup(keys, implicit_hyphen_keys, number_key, numbers)
        self.assertEqual(Stroke.KEYS, tuple(keys))
        self.assertEqual(Stroke.KEYS_IMPLICIT_HYPHEN, set(implicit_hyphen_keys))
        self.assertEqual(Stroke.KEYS_LETTERS, ''.join(keys).replace('-', ''))
        self.assertEqual(Stroke.KEY_FIRST_RIGHT_INDEX, keys.index('-E'))
        self.assertEqual(Stroke.NUMBER_KEY, number_key)
        self.assertEqual(Stroke.NUMBER_TO_KEY, {
            v.replace('-', ''): (k.replace('-', ''), keys.index(k))
            for k, v in numbers.items()
        })


class StrokeTestCase(unittest.TestCase):

    def setUp(self):
        class Stroke(BaseStroke):
            pass
        Stroke.setup(
            '''
            #
            S- T- K- P- W- H- R-
            A- O-
            *
            -E -U
            -F -R -P -B -L -G -T -S -D -Z
            '''.split(),
            'A- O- * -E -U'.split(),
            '#', {
                'S-': '1-',
                'T-': '2-',
                'P-': '3-',
                'H-': '4-',
                'A-': '5-',
                'O-': '0-',
                '-F': '-6',
                '-P': '-7',
                '-L': '-8',
                '-T': '-9',
            }
        )
        self.Stroke = Stroke

    def test_new(self):
        for test in (
            '''
            #
            #
            #
            0b00000000000000000000001
            False
            ''', '''
            T- -B -P S-
            S- T- -P -B
            ST-PB
            0b00000011000000000000110
            False
            ''', '''
            O- -E A-
            A- O- -E
            AOE
            0b00000000000101100000000
            False
            ''', '''
            -Z *
            * -Z
            *Z
            0b10000000000010000000000
            False
            ''', '''
            -R R-
            R- -R
            R-R
            0b00000000100000010000000
            False
            ''', '''
            S- -P O- # T-
            # S- T- O- -P
            120-7
            0b00000001000001000000111
            True
            ''', '''
            1- 2- 0- -7
            # S- T- O- -P
            120-7
            0b00000001000001000000111
            True
            ''', '''
            -L -F
            -F -L
            -FL
            0b00000100010000000000000
            False
            '''
        ):
            args = [a.strip() for a in test.strip().split('\n')]
            in_keys = args[0].split()
            keys = args[1].split()
            rtfcre = args[2]
            value = stroke_integer_type(args[3], 2)
            isnumber = eval(args[4])
            for init_arg in (in_keys, rtfcre, value):
                s = self.Stroke(init_arg)
                self.assertEqual(s, value,
                                 msg='Stroke(%r)=%#x, expected %#x' % (
                                     init_arg, stroke_integer_type(s), value
                                 ))
                self.assertEqual(s.keys(), keys,
                                 msg='Stroke(%r).keys()=%r, expected %r' % (
                                     init_arg, s.keys(), keys
                                 ))
                self.assertEqual(s.isnumber(), isnumber,
                                 msg='Stroke(%r).isnumber()=%r, expected %r' % (
                                     init_arg, s.isnumber(), isnumber
                                 ))
                self.assertEqual(str(s), rtfcre,
                                 msg='str(Stroke(%r))=%r, expected %r' % (
                                     init_arg, str(s), rtfcre
                                 ))

    def test_hash(self):
        for test in (
            '#       0b00000000000000000000001',
            'ST      0b00000000000000000000110',
            'STK     0b00000000000000000001110',
            '*       0b00000000000010000000000',
            '-PB     0b00000011000000000000000',
            'AOE     0b00000000000101100000000',
            'R-R     0b00000000100000010000000',
            'R-F     0b00000000010000010000000',
            'APBD    0b01000011000000100000000',
        ):
            rtfcre, value = test.split()
            s = self.Stroke(rtfcre)
            self.assertEqual(hash(s), stroke_integer_type(value, 2))

    def test_cmp(self):
        self.assertTrue(self.Stroke('#') < 'ST')
        self.assertTrue(self.Stroke('ST') <= 'STK')
        self.assertTrue(self.Stroke('STK') <= 'STK')
        self.assertTrue(self.Stroke('STK') == 'STK')
        self.assertTrue(self.Stroke('*') != 'R-R')
        self.assertTrue(self.Stroke('-PB') > 'AOE')
        self.assertTrue(self.Stroke('R-R') >= 'R-F')
        self.assertTrue(self.Stroke('APBD') >= 'APBD')

    def test_sort(self):
        unsorted_strokes = [
            self.Stroke(s)
            for s in '''
            AOE
            ST-PB
            *Z
            #
            R-R
        '''.split()
        ]
        sorted_strokes = [
            self.Stroke(s)
            for s in '''
            #
            ST-PB
            R-R
            AOE
            *Z
        '''.split()
        ]
        self.assertEqual(list(sorted(unsorted_strokes)), sorted_strokes)


if __name__ == '__main__':
    unittest.main()
